// Moebius 05
// Versione Finale Si costruisce un anello a partire da una qualunque sezione
// Si introduce una nuova classe per la gestione delle sezioni

#ifdef WIN32
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#endif
#include <GL/gl.h>	
#include <GL/glu.h>	
#include <stdlib.h>
#include <time.h>
#include <SDL.h>
#include <QImage.h>
#include <QMessageBox.h>
#include <qapplication.h>
#include <QGL.h>
#include <vector>

#include <vcg/Point3.h>
#include <vcg/Matrix44.h>
#include "CIMoebius.h"

using namespace vcg;
using namespace std;

CIMoebius ring;

void DrawGLScene()
{
	
	static int LastT=clock();
	static float CurAngle=0;
  int CurT=clock();
	float DeltaT=(CurT-LastT)/1000.0;  // elapsed time in secs.
	if(DeltaT < 0.01) return;
	LastT=CurT;
	float AngularSpeed = 90;  // in degree/sec
	CurAngle+=AngularSpeed*DeltaT;

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	
	float spec[4]={.4,.4,.4,1};
	float shin = 1;
	glMaterialfv(GL_FRONT_AND_BACK,GL_SPECULAR , spec);
	glMaterialf(GL_FRONT_AND_BACK,GL_SHININESS , shin);
	
	glColorMaterial(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE );
	glEnable(GL_COLOR_MATERIAL);
	
	float lightpos[4]={0,0,1,0};
	glLoadIdentity();
	glLightfv(GL_LIGHT0,GL_POSITION,lightpos);
	gluLookAt(2,5,12,0,0,0,0,1,0);
	glRotatef(-CurAngle,0,1,0);

  glColor3f(1,.4,.4);
  ring.m.Draw();	
	SDL_GL_SwapBuffers();
}

// Inizializzazione dello stato di OpenGL
void myGLInit()
{
	// Abilita il la possibilita' di cambiare il alcune componenti colore del 
	// materiale tramite un semplice glcolor, anzich fare glMaterial(...)
	glColorMaterial(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE);
	glEnable(GL_COLOR_MATERIAL);

	// Abilita il calcolo dell'illuminazione
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);

	// anche le normali attraversano la pipeline, quindi devono essere  
	// rinormalizzate, prima del calcolo dell'illuminazione, altrimenti 
	// il calcolo dell'illuminazione viene sbagliato.
	glEnable(GL_NORMALIZE); 
	
	// Abilita il test sullo zbuffer e quindi l'eliminazione delle 
	// superfici nascoste
	glEnable(GL_DEPTH_TEST); 	

	// Disabilita lo shading di gouraud per le facce dei poligoni disegnati
	glShadeModel(GL_SMOOTH);

	QImage tt; 	tt.load("Texture.png");
	QImage tx = QGLWidget::convertToGLFormat ( tt);
	printf("QImage loaded %i %i \n",tt.size().width(),tt.size().height());

	GLuint texName;
	glGenTextures(1, &texName);
	glBindTexture(GL_TEXTURE_2D, texName);
	glTexImage2D( GL_TEXTURE_2D, 0, 3, tx.width(), tx.height(), 0, GL_RGBA, GL_UNSIGNED_BYTE, tx.bits() );
	gluBuild2DMipmaps(GL_TEXTURE_2D, 3, tx.width(), tx.height(), GL_RGBA, GL_UNSIGNED_BYTE, tx.bits() );
	glEnable(GL_TEXTURE_2D);
	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);


}

void myGLReshapeFunc(GLsizei w, GLsizei h)
{
	glMatrixMode (GL_PROJECTION);   
	glLoadIdentity ();
	float ratio=(float)w/(float)h;
  gluPerspective(45,ratio,3,20);
	glViewport (0, 0, (GLsizei) w, (GLsizei) h); 
	glMatrixMode (GL_MODELVIEW);
	myGLInit();
}


int main(int argc, char **argv) 
{  
	QApplication QApp(argc, argv );
	
  /* Initialize SDL for video output */
  if ( SDL_Init(SDL_INIT_VIDEO) < 0 ) {
    fprintf(stderr, "Unable to initialize SDL: %s\n", SDL_GetError());
    exit(1);
  }
  /* Specify the requirements for the gl window*/
	SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, 24);   
  
	/* Create a 640x480 OpenGL screen */
  if ( SDL_SetVideoMode(640, 480, 0, SDL_OPENGL |SDL_FULLSCREEN) == NULL ) {
    fprintf(stderr, "Unable to create OpenGL screen: %s\n", SDL_GetError());
    SDL_Quit();
    exit(2);
  }

	CIMoebius::myShape s;
  s.BuildNGon(3,1);
  ring.GenerateMoebiusRing(s,36,3,120);

	/* Loop, drawing and checking events */
	int done = 0;
	int ret;
  myGLReshapeFunc(640,480);  
	while ( ! done ) 
	{
    SDL_Event event;
	  ret=SDL_PollEvent(&event);
		if(ret)
			switch(event.type)
			{  
						case SDL_QUIT    : done = 1;  
															break ;
						case SDL_KEYDOWN : if ( event.key.keysym.sym == SDLK_ESCAPE ) 
																				done = 1;
															 if ( event.key.keysym.sym == SDLK_SPACE ) 
																					QMessageBox::information(0,"asdfads","Asdfadsf");
	

															break;
						case SDL_VIDEORESIZE	: SDL_SetVideoMode(event.resize.w,event.resize.h, 0, SDL_OPENGL | SDL_FULLSCREEN);
																		myGLReshapeFunc(event.resize.w,event.resize.h); 
																		break;
			}  
		else DrawGLScene();
  }
  SDL_Quit();
  return 1;
}
