// Moebius 05
// Versione Finale Si costruisce un anello a partire da una qualunque sezione
// Si introduce una nuova classe per la gestione delle sezioni

#ifdef WIN32
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#endif
#include <GL/gl.h>	
#include <GL/glu.h>	
#include <stdlib.h>
#include <time.h>
#include <vcg/Point3.h>
#include <vcg/Matrix44.h>
#include <vector>
using namespace vcg;
using namespace std;
#include "CISG.h"

#include "CIMoebius.h"


void CIMoebius::myFace::ComputeNormal()
{
	n=(v[1]->p-v[0]->p)^(v[2]->p-v[0]->p);
	n.Normalize();
}
CIMoebius::myFace::myFace(	myVertex *v0, myVertex *v1, myVertex *v2)
{
	v[0]=v0;v[1]=v1;v[2]=v2;
	ComputeNormal();
}
// costruisce un poligono regolare di n lati e raggio rad
void CIMoebius::myShape::BuildNGon(int n, double rad)
{
	poly.resize(n*2);
	double a =2.0*M_PI/double(n); 

	for(int i=0;i<n;i++) {
		poly[i*2+0].p=Point3f( rad*cos(i*a - a/2) , rad*sin(i*a - a/2) ,0); 
		poly[i*2+1].p=Point3f( rad*cos(i*a + a/2) , rad*sin(i*a + a/2) ,0); 
	}
	ComputeNormals();
}

// Dato un poligono sul piano xy calcola le normali ai vertici
// due vertici identici consecutivi significano crease (stessa geometria ma normali diverse)
void CIMoebius::myShape::ComputeNormals()
{
	int i;
	int sz=poly.size();
	for(i=0;i<sz;++i)
	{
		// la normale del vertice v_i e data dalla media delle normali degli edge [v_(i-1),v_(i)] e [v_(i),v_(i+1)] 
   Point3f eprev = poly[ i      ].p - poly[(i-1+sz)%sz].p;
   Point3f enext = poly[(i+1)%sz].p - poly[   i       ].p;
	 // per trovare la normale basta ruotare di 90 gradi intorno all'asse z cioe' scambiare x e y cambiando di segno alla y
   Point3f nprev,nnext;
	 nprev.x() = eprev.y();
	 nprev.y() = -eprev.x();
	 nprev.z() = 0;
	 nnext.x() =  enext.y();
	 nnext.y() =  -enext.x();
	 nnext.z() = 0;
	 
	 poly[i].n=Point3f(0,0,0);
	 // se uno degli edge era nullo non si considera
	 if(eprev.SquaredNorm()!=0) poly[i].n+=nprev;
	 if(enext.SquaredNorm()!=0) poly[i].n+=nnext;
	 
	 poly[i].n.Normalize();
	}
}

void CIMoebius::myMesh::Draw()
{
	glBegin(GL_TRIANGLES);
	vector<myFace>::iterator fi;
	for(fi=face.begin();fi!=face.end();++fi){
		glNormal(  (*fi).v[0]->n);
		glTexCoord((*fi).v[0]->t);
		glVertex(  (*fi).v[0]->p);
		glNormal(  (*fi).v[1]->n);
		glTexCoord((*fi).v[1]->t);
		glVertex(  (*fi).v[1]->p);
		glNormal(  (*fi).v[2]->n);
		glTexCoord((*fi).v[2]->t);
		glVertex(  (*fi).v[2]->p);
	}
	glEnd();
}


// TwistDeg indica di quanti gradi si deve avvolgere

void CIMoebius::GenerateMoebiusRing(myShape &s, int step, float r, float twistDeg)
{
  m.clear();
	float angle=-360.0f/step;
	float angletwist=(twistDeg)/step;
	int i,j;
	float perimeter=0;
	for(j=0;j<s.poly.size();++j)
		perimeter+=Distance(s.poly[j].p,s.poly[(j+1)%s.poly.size()].p);
	for(i=0;i<=step;++i)
	{
		Matrix44f mm,rr,tt,tw;
		rr.Rotate(i*angle,Point3f(0,1,0));
		tt.Translate(r,0,0);
		tw.Rotate(i*angletwist,Point3f(0,0,1));
		mm=rr*tt*tw;
		float curlen=0;
		for(j=0;j<s.poly.size();++j){
			m.vert.push_back(myVertex(mm*s.poly[j].p, mm*s.poly[j].n4(),Point3f(float(i)/step, curlen/perimeter,0)));
			curlen+=Distance(s.poly[j].p,s.poly[(j+1)%s.poly.size()].p);
		}
	}
	int sz=m.vert.size();
	int ssz=s.poly.size();
	for(i=0;i<step;++i)
	{
		for(int j=0;j<s.poly.size();++j) 
		{
			// Aggiungo triangoli solo se i due vertici consecutivi della shape non sono uguali 
			if(s.poly[j].p != s.poly[(j+1)%ssz].p) 
			{
				m.face.push_back(myFace(&m.vert[((i+0)*ssz+(j+0)%ssz)%sz] ,
																&m.vert[((i+0)*ssz+(j+1)%ssz)%sz] ,
																&m.vert[((i+1)*ssz+(j+1)%ssz)%sz] ));
				m.face.push_back(myFace(&m.vert[((i+0)*ssz+(j+0)%ssz)%sz] ,
																&m.vert[((i+1)*ssz+(j+1)%ssz)%sz] ,
																&m.vert[((i+1)*ssz+(j+0)%ssz)%sz] ));
			}
		}
	}
}

void CIMoebius::glDraw(float)
{
	m.Draw();
}
